/**
 * jsprec.js v1.0
 * Implements Number.add() and Number.mult() without the floating precision errors like these:
 *   0.1*0.2 = 0.020000000000000004
 *   0.1+0.2 = 0.30000000000000004
 * http://nlacsoft.net
 * no dependencies, BSD license
 */
;(function(ns){
"use strict";

	var pad0 = "000000000000000000000000";
	
	/**
	 * Removes the sign + ending zeros properly
	 */
	function _normalize(s) {
		return (""+s).replace(/^[+-]+/,"")
			.replace(/^\./,"0.")
			.replace(/(.*\..*)0+$/,"$1")
			.replace(/\.$/,"");
	};
	
	/**
	 * Returns the minimum positions need the number to shift with, to be integer
	 */
	function _getShift(x) {
		var s = ""+x, pos = s.indexOf(".");
		if (pos<0)
			return 0;
		s = _normalize(s);
		pos = s.indexOf(".");
		if (pos<0)
			return 0;
		return s.length - pos - 1;
	};
	
	/**
	 * Shifts decimal point properly
	 * 
	 * -0.0023000,5  -> -230
	 * 0.0023000,-2  -> 0.000023
	 * 134,3         -> 134000
	 */
	function _shift(x, nShift) {
		if (nShift==0)
			return x;
		var s = ""+x, pos = s.indexOf(".");
		s = s.replace(/^[+-]*/,"");
		if (pos<0)
			s += ".0";
		pos = s.indexOf(".");
		s = pad0 + s + pad0;
		pos += (pad0.length + nShift);
		
		s = s.replace(/\./,"");
		s = _normalize(s.substr(0,pos) + "." + s.substr(pos));

		if (x<0)
			s = "-" + s;
		return Number(s);
	};
	
	/**
	 * Multiplication
	 */
	ns.mult = function(x,y) {
		var sx = _getShift(x), sy = _getShift(y);
		return _shift(_shift(x, sx) * _shift(y, sy), -sx - sy);
	};
	
	/**
	 * Addition
	 */
	ns.add = function(x,y) {
		var s = Math.max(_getShift(x), _getShift(y));
		return _shift(_shift(x, s) + _shift(y, s), -s);
	};

})(Number);